using FitConnect.Interfaces.Sender;
using FitConnect.Models;
using FitConnect.Services.Models.v1.Submission;
using Microsoft.Extensions.Logging;

namespace FitConnect;

public class EncryptedSubmissionBuilder : IEncryptedBuilderWithData,
    IEncryptedBuilderWithMetadata, IEncryptedBuilderWithAttachments, IEncryptedBuilderWithDestination, IEncryptedBuilderWithService {
    private Dictionary<string, string>? EncryptedAttachments;
    private string? EncryptedData;
    private string? EncryptedMetadata;
    
    private SendableEncryptedSubmission _submission;

    public static IEncryptedBuilderWithDestination WithDestination(string destinationId) {
        return new EncryptedSubmissionBuilder(destinationId);
    }
    
    public IEncryptedBuilderWithService WithServiceType(string serviceName, string leikaKey) {
        _submission.ServiceName = serviceName;
        _submission.LeikaKey = leikaKey;
        return this;
    }

    public IEncryptedBuilderWithData WithEncryptedData(string data) {
        _submission.Data = data;
        return this;
    }

    public IEncryptedBuilderWithMetadata WithEncryptedMetaData(string metaData) {
        _submission.Metadata = metaData;
        return this;
    }


    public IEncryptedBuilderWithAttachments WithEncryptedAttachments(
        Dictionary<string, string> attachments) {
        _submission.Attachments = attachments;
        return this;
    }


    public SendableEncryptedSubmission Build() {
        return _submission;
    }

    private protected EncryptedSubmissionBuilder(string destinationId) {
        _submission = new SendableEncryptedSubmission(destinationId);
    }
}


public interface IEncryptedBuilderWithDestination {
    IEncryptedBuilderWithService WithServiceType(string serviceName, string leikaKey);
}
public interface IEncryptedBuilderWithService {
    IEncryptedBuilderWithMetadata WithEncryptedMetaData(string metaData);
}

public interface IEncryptedBuilderWithMetadata {
    IEncryptedBuilderWithData WithEncryptedData(string data);
}

public interface IEncryptedBuilderWithData {
    IEncryptedBuilderWithAttachments WithEncryptedAttachments(
        Dictionary<string, string> attachments);
}

public interface IEncryptedBuilderWithAttachments {
    SendableEncryptedSubmission Build();
}
